@@ -38,7 +38,7 @@ except ImportError:
 __author__ = 'Igor Davydenko'
 __license__ = 'BSD License'
 __script__ = 'bootstrapper'
-__version__ = '0.3.1'
+__version__ = '0.4'
 
 
 IS_PY3 = sys.version_info[0] == 3
@@ -202,12 +202,15 @@ def install(env, requirements, args, ignore_activated=False, quiet=False):
     if not quiet:
         print('== Step 2. Install {0} =='.format(label))
 
-    pip_cmd(env, ('install', ) + args, ignore_activated, echo=not quiet)
+    result = not pip_cmd(env,
+                         ('install',) + args,
+                         ignore_activated,
+                         echo=not quiet)
 
     if not quiet:
         print()
 
-    return True
+    return result
 
 
 @error_handler
@@ -234,19 +237,27 @@ def main(*args):
 
     # Create virtual environment
     env_args = prepare_args(config['virtualenv'], bootstrap)
-    create_env(bootstrap['env'],
-               env_args,
-               bootstrap['recreate'],
-               bootstrap['ignore_activated'],
-               bootstrap['quiet'])
+    if not create_env(
+        bootstrap['env'],
+        env_args,
+        bootstrap['recreate'],
+        bootstrap['ignore_activated'],
+        bootstrap['quiet']
+    ):
+        # Exit if couldn't create virtual environment
+        return True
 
     # And install library or project here
     pip_args = prepare_args(config['pip'], bootstrap)
-    install(bootstrap['env'],
-            bootstrap['requirements'],
-            pip_args,
-            bootstrap['ignore_activated'],
-            bootstrap['quiet'])
+    if not install(
+        bootstrap['env'],
+        bootstrap['requirements'],
+        pip_args,
+        bootstrap['ignore_activated'],
+        bootstrap['quiet']
+    ):
+        # Exist if couldn't install requirements into venv
+        return True
 
     # Run post-bootstrap hook
     run_hook(bootstrap['hook'], bootstrap, bootstrap['quiet'])
@@ -331,7 +342,7 @@ def pip_cmd(env, cmd, ignore_activated=False, **kwargs):
         raise OSError('No pip found at {0!r}'.format(pip_path))
 
     with disable_error_handler():
-        return not run_cmd((pip_path, ) + cmd, **kwargs)
+        return run_cmd((pip_path, ) + cmd, **kwargs)
 
 
 def prepare_args(config, bootstrap):
@@ -395,39 +406,43 @@ def read_config(filename, args):
     sections = set(iterkeys(default))
 
     # Expand user and environ vars in config filename
+    is_default = filename == DEFAULT_CONFIG
     filename = os.path.expandvars(os.path.expanduser(filename))
 
     # Read config if it exists on disk
-    if os.path.isfile(filename):
-        parser = SafeConfigParser()
+    if not is_default and not os.path.isfile(filename):
+        print_error('Config file does not exist at {0!r}'.format(filename))
+        return None
 
-        try:
-            parser.read(filename)
-        except ConfigParserError:
-            print_error('Cannot parse config file at {0!r}'.format(filename))
-            return None
+    parser = SafeConfigParser()
 
-        # Apply config for each possible section
-        for section in sections:
-            if not parser.has_section(section):
-                continue
+    try:
+        parser.read(filename)
+    except ConfigParserError:
+        print_error('Cannot parse config file at {0!r}'.format(filename))
+        return None
 
-            items = parser.items(section)
+    # Apply config for each possible section
+    for section in sections:
+        if not parser.has_section(section):
+            continue
 
-            # Make auto convert here for integers and boolean values
-            for key, value in items:
+        items = parser.items(section)
+
+        # Make auto convert here for integers and boolean values
+        for key, value in items:
+            try:
+                value = int(value)
+            except (TypeError, ValueError):
                 try:
-                    value = int(value)
-                except (TypeError, ValueError):
-                    try:
-                        value = bool(strtobool(value))
-                    except ValueError:
-                        pass
+                    value = bool(strtobool(value))
+                except ValueError:
+                    pass
 
-                if section in converters and key in converters[section]:
-                    value = converters[section][key](value)
+            if section in converters and key in converters[section]:
+                value = converters[section][key](value)
 
-                config[section][key] = value
+            config[section][key] = value
 
     # Update config with default values if necessary
     for section, data in iteritems(default):
@@ -463,6 +478,8 @@ def run_cmd(cmd, echo=False, fail_silently=False, **kwargs):
     If ``echo`` show command to call and its output in STDOUT, otherwise hide
     all output.
     """
+    out, err = None, None
+
     if echo:
         cmd_str = cmd if isinstance(cmd, string_types) else ' '.join(cmd)
         kwargs['stdout'], kwargs['stderr'] = sys.stdout, sys.stderr
@@ -477,6 +494,11 @@ def run_cmd(cmd, echo=False, fail_silently=False, **kwargs):
         if fail_silently:
             return False
         print_error(str(err) if IS_PY3 else unicode(err))  # noqa
+    finally:
+        if out:
+            out.close()
+        if err:
+            err.close()
 
     if retcode and echo and not fail_silently:
         print_error('Command {0!r} returned non-zero exit status {1}'.
